unit UCustomCGIApplication;

{$mode delphi}

interface

uses
  Classes, SysUtils, UHTTPServerCGI, Contnrs, USqlDatabase, UConfig, DateUtils,
  UXmlClasses, UHtmlClasses, UCore, UStringListEx, UHTTPServer;

type
  TCustomCGIApplication = class;

  TPageProducer = procedure(Body: TCustomCGIApplication);

  TRegistredPage = class
    Name: string;
    Producer: TPageProducer;
  end;

  { TCustomCGIApplication }

  TCustomCGIApplication = class(THTTPServerCGI)
  private
    procedure Footer;
    procedure InitDatabase;
    procedure Show(Content: string);
    procedure TopMenu;
  public
    TimeStart: TDateTime;
    Load: string;
    Unload: string;
    GlobalTitle: string;
    ShowRuntimeInfo: Boolean;
    Charset: string;
    Admin: string;
    AdminEmail: string;
    Style: string;
    Keywords: string;
    HtmlDocument: THtmlDocument;
    Pages: TObjectList;
    Database: TSqlDatabase;
    PageName: string;
    ModuleName: string;
    BaseURL: string;
    procedure Execute(HandlerData: THTTPHandlerData);
    constructor Create;
    destructor Destroy; override;
    procedure RegisterPage(Name: string; Producer: TPageProducer);
  end;


implementation

resourcestring
  SPageNotFound = 'Page not found';

{ TCustomCGIApplication }

procedure TCustomCGIApplication.RegisterPage(Name: string; Producer: TPageProducer);
var
  NewRegistredPage: TRegistredPage;
begin
  NewRegistredPage := TRegistredPage.Create;
  NewRegistredPage.Name := Name;
  NewRegistredPage.Producer := Producer;
  Pages.Add(NewRegistredPage);
end;


procedure TCustomCGIApplication.InitDatabase;
var
  DbRows: TDbRows;
begin
  Database := TSqlDatabase.Create;
  with Database do begin
    Hostname := DatabaseHostName;
    Database := DatabaseDatabase;
    UserName := DatabaseUserName;
    Password := DatabasePassword;
    Connect;
  end;
  DbRows := Database.Query('SET NAMES utf8');
  DbRows.Free;
end;

procedure TCustomCGIApplication.Execute(HandlerData: THTTPHandlerData);
var
  I: Integer;
  TitleTag: THtmlString;
begin
  with HandlerData do begin
  Style := ConfigStyle;
  BaseURL := ConfigBaseURL;

  HtmlDocument.ContentLanguage := 'cs';
  GlobalTitle := 'ZděchovNET';
  HtmlDocument.Styles.Add(NavigationLink('/Style/' + Style + '/Style.css'));
  HtmlDocument.Scripts.Add(NavigationLink('/Style/' + Style + '/Global.js'));
  HtmlDocument.Scripts.Add(NavigationLink('/Style/' + Style + '/jquery.js'));

  if Request.QueryParts.Count > 0 then PageName := Request.QueryParts[0]
    else PageName := '';
  I := 0;
  while (I < Pages.Count) and (TRegistredPage(Pages[I]).Name <> PageName) do Inc(I);
  if (I < Pages.Count) then begin
    TitleTag := THtmlString.Create;
    HtmlDocument.Body.SubItems.Add(TitleTag);
    TopMenu;
    TRegistredPage(Pages[I]).Producer(Self);
    TitleTag.Text := '<div class="TitlePanel"><span class="GlobalTitle">' + GlobalTitle +
    '</span> - ' + HtmlDocument.Title + '</div>';
    HtmlDocument.Title := GlobalTitle + ' - ' + HtmlDocument.Title;
    with HtmlDocument.AsXmlDocument do try
      Formated := ConfigFormatHTML;
      Response.Stream.WriteString(AsString);
    finally
      Free;
    end;
  end else Response.Stream.WriteString(SPageNotFound);
  end;
end;

constructor TCustomCGIApplication.Create;
begin
  inherited;
  Database := TSqlDatabase.Create;
  InitDatabase;
  Pages := TObjectList.Create;
  HtmlDocument := THtmlDocument.Create;
  OnRequest := Execute;
end;

destructor TCustomCGIApplication.Destroy;
begin
  HtmlDocument.Free;
  Database.Free;
  Pages.Free;
  inherited Destroy;
end;

procedure TCustomCGIApplication.Footer;
var
  Tag: TXMLTag;
  AdminTag: TXMLTag;
  EmailTag: TXMLTag;
  ExecutionTimeTag: TXMLTag;
  UsedMemoryTag: TXMLTag;
  TextNode: TXmlString;
begin
  with TXmlTag(HtmlDocument.Body.SubItems[HtmlDocument.Body.SubItems.Add(TXmlTag.Create)]) do begin
    Name := 'ul';
    Attributes.Values['class'] := 'Footer';
    with TXmlTag(SubElements[SubElements.Add(TXmlTag.Create)]) do begin
      Name := 'li';
      with TXmlString(SubElements[SubElements.Add(TXmlString.Create)]) do begin
        Text := Admin;
      end;
    end;
    with TXmlTag(SubElements[SubElements.Add(TXmlTag.Create)]) do begin
      Name := 'li';
      with TXmlString(SubElements[SubElements.Add(TXmlString.Create)]) do begin
        Text := AdminEmail;
      end;
    end;
    if ShowRuntimeInfo then begin
      with TXmlTag(SubElements[SubElements.Add(TXmlTag.Create)]) do begin
        Name := 'li';
        with TXmlString(SubElements[SubElements.Add(TXmlString.Create)]) do begin
          Text := 'Doba generování: ' +
            FloatToStr(Round(((Now - TimeStart) / OneMillisecond) * 100) / 100) + ' s / '; // + ini_get('max_execution_time') + ' s';
        end;
      end;
      with TXmlTag(SubElements[SubElements.Add(TXmlTag.Create)]) do begin
        Name := 'li';
        with TXmlString(SubElements[SubElements.Add(TXmlString.Create)]) do begin
          //Text := 'Použitá paměť: '  + System.PrefixMultiplier.AddPrefixMultipliers(memory_get_peak_usage(FALSE), 'B').' / '.ini_get('memory_limit').'B';
        end;
      end;
    end;
  end;
end;

procedure TCustomCGIApplication.Show(Content: string);
begin
  //HtmlDocument.Title := ;
(*  BodyParam := '';
  if Load <> '' then BodyParam := BodyParam + ' onload="' + Load + '"';
  if Unload <> '' then BodyParam := BodyParam + ' onunload="' + Unload + '"';
  Document.Encoding := Charset;
  Document.Formated := FormatHTML;*)


(*
  Title := GlobalTitle;
  if Title <> '' then Title := Title + ' - ' + Title;
  Document.TitleTag.SubElements := Title;
  Title := '<span class="GlobalTitle">' + GlobalTitle + '</span>';
  if Title <> '' then Title := Title + ' - ' + $this.Title;

  Keywords := TXMLTag.Create('meta');
  Keywords.Attributes := array('name' => 'keywords', 'content' => implode(',', $this->Keywords));
  Keywords.ShringEmpty := False;
  Document.HeadTag.SubElements.Add(Keywords);

  Document.BodyTag.SubElements := array('<div class="TitlePanel">' + Title + '</div>',
  TopMenu, PageMenu, Content, Footer);
  Result := Document.GetOutput;        *)
end;

procedure TCustomCGIApplication.TopMenu;
begin
  with THtmlString(HtmlDocument.Body.SubItems[HtmlDocument.Body.SubItems.Add(THtmlString.Create)]) do begin
    Text := '<div class="Navigation">';
    // Visitor
    Text := Text + '<ul class="MenuItem">' +
      '<li>' + MakeLink('O síti', NavigationLink('/')) + '</li>' +
      '<li>' + MakeLink('Internet', NavigationLink('/internet/')) + '</li>' +
      '<li>' + MakeLink('Hosting', NavigationLink('/hosting/')) + '</li>' +
      '<li>' + MakeLink('VoIP', NavigationLink('/voip/')) + '</li>' +
      '<li>' + MakeLink('Historie', NavigationLink('/historie/')) + '</li>' +
      '<li>' + MakeLink('Dokumenty', NavigationLink('/dokumenty/')) + '</li>' +
      '<li>' + MakeLink('Odkazy', NavigationLink('/odkazy/')) + '</li>' +
      '<li>' + MakeLink('Kontakt', NavigationLink('/kontakt/')) + '</li>' +
      '<li><a href="https://mail.zdechov.net/">Pošta</a></li>' +
      '<li><a href="http://wiki.zdechov.net/">Wiki</a></li>' +
      '</ul><ul class="MenuItem2"><li>&nbsp;</li>';
(*    if System.UserOnline.User = System.UserOnline.AnonymousUserId then begin
      Text := Text + '</ul>' +
        '<ul class="MenuItem2">' +
        '<li>' + MakeLink('Přihlášení', NavigationMakeLink('User', 'Login')) + '</li>';
      //$Output .= '<li>'.$this->System->HTML->MakeLink($this->System->Translate('UserRegistration'), $this->System->MakeLink('UserList', 'Register')).'</li>';
    end else begin
      Text := Text + '</ul>' +
        '<ul class="MenuItem2">' +
        '<li>' + System.User.FullName + '</li>' +
        '<li>' + MakeLink('Odhlášení', NavigationMakeLink('User', 'Logout')) + '</li>' +
        '<li>' + MakeLink('Profil', NavigationMakeLink('User', 'View',
        array('Id' => $this->System->User->Id))).'</li>';
    end;
    *)
    Text := Text + '</ul></div>';
  end;
end;


end.

