unit UHTTPServerCGI;

{$mode delphi}

interface

uses
  Classes, SysUtils, UHTTPServer;

type

  { THTTPServerCGI }

  THTTPServerCGI = class(THTTPServer)
  public
    EnvVars: TStringList;
    procedure Run; override;
    constructor Create;
    destructor Destroy; override;
    procedure SysInfo;
  end;

implementation

{ THTTPServerCGI }

constructor THTTPServerCGI.Create;
begin
  inherited;
  EnvVars := TStringList.Create;
end;

destructor THTTPServerCGI.Destroy;
begin
  EnvVars.Free;
  inherited Destroy;
end;

procedure THTTPServerCGI.Run;
var
  I: Integer;
  HandlerData: THTTPHandlerData;
begin
  HandlerData := THTTPHandlerData.Create;
  with HandlerData do try
    // Load environment variables
    for I := 0 to GetEnvironmentVariableCount - 1 do begin
      EnvVars.Add(GetEnvironmentString(I));
    end;

    // Process cookies
    if Request.Headers.IndexOfName('Cookie') <> -1 then
      Request.Cookies.Parse(Request.Headers.Values['Cookie']);

    // Parse query string
    if Length(EnvVars.Values['QUERY_STRING']) > 0 then
    if EnvVars.Values['QUERY_STRING'][Length(EnvVars.Values['QUERY_STRING'])] = '/' then
      EnvVars.Values['QUERY_STRING'] := Copy(EnvVars.Values['QUERY_STRING'], 1,
        Length(EnvVars.Values['QUERY_STRING']) - 1);
    Request.QueryParts.Explode('/', EnvVars.Values['QUERY_STRING']);
    if Pos('?', EnvVars.Values['REQUEST_URI']) > 0 then
      Request.Query.Parse(Copy(EnvVars.Values['REQUEST_URI'],
        Pos('?', EnvVars.Values['REQUEST_URI']) + 1,
        Length(EnvVars.Values['REQUEST_URI'])));

    Response.Stream.Clear;
    Response.Headers.Values['Content-type'] := 'text/html';

    if Assigned(OnRequest) then OnRequest(HandlerData)
      else raise EEmptyHTTPHandler.Create(SEmptyHTTPHandler);

    with Response do begin
      // Generate headers
      for I := 0 to Headers.Count - 1 do begin
        WriteLn(Headers.Names[I] + ': ' + Headers.ValueFromIndex[I]);
      end;

      // Generate cookies
      for I := 0 to Cookies.Count - 1 do
        Headers.Add('Set-Cookie' + Headers.NameValueSeparator + Cookies.Names[I] + '=' + Cookies.ValueFromIndex[I]);
        // + ';path=/;expires=' + RFC822DateTime(Now);

      WriteLn; // Empty line header separator

      // Emit page content
      Stream.Position := 0;
      WriteLn(Stream.ReadString);
    end;
  finally
    HandlerData.Free;
  end;
end;

procedure THTTPServerCGI.SysInfo;
var
  I: Integer;
begin
(*  Output.Add('<h4>Environment variables:</h4>');
  Output.Add('<table border="1">');
  for I := 0 to EnvVars.Count - 1 do begin
    Output.Add('<tr><td>' + EnvVars.Names[I] + '</td><td>' +
      EnvVars.ValueFromIndex[I] + '</td></tr>');
  end;
  Output.Add('</table>');*)
end;

end.

